;###############################
;#  LOADER FOR DRUID AND DROID #
;# (entry for CPCRetrodev 2022)#
;#    cassette tape version    #
;###############################


org #9000

nolist
scraddr_lion equ #f658
lion_decompressed equ #4000

	call #bc14					;cls
	call set_colors_leosoft
	call decompress_lion
	call anim_swoosh
	call sound_chords
	call anim_logo
	call write_presents
	ld a,1:call #bc6b			;no cas messages

	ld hl,filename_title		;title screen
	ld b,7:ld de,#A000			;length of filename & buffer
	call #bc77					;cas in open
	jp nc,0						;reset if problem
	ld hl,#6000					;load compressed title to #6000
	push hl
	call #bc83					;cas in direct
	call #bc7a					;cas in close
	pop hl						;restore address
	ld de,#1000					;destination
	di
	call DecompressLZSA2		;decompress title screen
	ei
	xor a:call #bc0e			;set mode 0
	ld a,16						;set colors for title (and game)
	ld hl,colors_title
	call set_colors
	ld hl,#1000:ld de,#c000
	ld bc,#3fff:ldir			;copy title to screen memory

	ld bc,#0109:call #BC38		;set border blue/green
	ld hl,#0101:call #BC3E		;flash fast

	ld hl,filename_game			;load compressed GAME
	ld b,7:ld de,#A000
	call #bc77:jp nc,0			;cas in open
	push bc
	ld hl,#80				;load compressed game to #0080
	call #bc83				;cas in direct
	call #bc7a				;cas in close
	ld hl,#c7d0				;when loaded, show "press key"
	ld a,6
		copy_presskey_label:
		ld d,h:ld e,#a0
		ld bc,17:ldir
		ld bc,#800-17:add hl,bc
		dec a
		jr nz,copy_presskey_label

	ld bc,#0:call #BC38			;set border black

	call #bb03					;reset kbd buffer
	call #bb06					;wait key

	di
	ld bc,#bc01:out (c),c		;make screen 0 bytes wide to hide what's
	inc b:dec c:out (c),c		;going on but w/o resetting the palette
	pop bc
	newstack equ $			;we don't need everything until here anymore
	ld sp,newstack			;but to make place for the compressed game,
					;we have to move the stack
	ld hl,#80
	ld de,place_for_compressed_game
	push hl:push de				;backup src & dest
	ldir					;copy compressed game to the 2 high banks

	pop hl:pop de
	call DecompressLZSA2			;and decompress it to #0080 again

	ld hl,decompressLZSA2:ld (hl),0
	ld de,decompressLZSA2+1
	ld bc,#ffff-decompressLZSA2
	ldir						;clear everything above here

	jp #80						;and start game


;this is the lzsa decompression tool by Emmanuel Marty
;from https//github.com/emmanuel-marty/lzsa
read "lzsa.asm"


filename_game:
db "DD2.BIN"


place_for_compressed_game:

filename_title:
db "DD1.BIN"

decompress_lion:
	ld hl,lion_lzsa			;source
	ld de,lion_decompressed	;dest
	di
	call decompresslzsa2	;decompress lion image
	ei
	ret

drawlogo:
	ld hl,scraddr_lion-2
	call wait
	ld ix,parts
		logo_linesloop:
		push de
		push hl
		ld b,3	;lines
			logo_outerloop:
			push bc
			ld b,7	;symbols
				logo_innerloop:
				inc hl
				ld a,(de)
				rra
				call drawlogo_byte
				ld a,(de)
				rla
				rla
				rla
				call drawlogo_byte
				inc de
				djnz logo_innerloop
			ld c,45
			add hl,bc
			pop bc
			djnz logo_outerloop
		pop hl
		pop de
		call #bc26
		cp #30
		ret z
		inc ix
		jr logo_linesloop

	drawlogo_byte:
		and #38
		ld (symbol_offset+2),a
		symbol_offset:
		ld a,(ix+000)
			ld (hl),a
			rld
			inc hl
			ld (hl),a
			inc hl
	ret

anim_swoosh:
	ld c,189				;height and
	ld de,#ffee				;bitmask for
	call swoosh				;brown swoosh
	ld c,166				;height and
	ld de,0					;bitmask for
	;fall through			;black (erase) swoosh
	swoosh:
	ld hl,#c017
	swoosh2:
		push hl
		halt
		ld b,7
			swoosh1:
			call swooshhead
			inc hl
			djnz swoosh1
		pop hl
		call #bc26
		dec c
		jr nz,swoosh2
	ret

	swooshhead:
	call $+3
	ld (hl),d
	inc hl
	ld (hl),e
	inc hl
	ret

set_colors_leosoft:
	ld a,4					;4 colors in mode 1
	ld hl,colors_leosoft
	;fall through
set_colors:
	ld b,(hl)
	ld c,b
	inc hl
	push af
	push hl
	dec a
	call #bc32
	pop hl
	pop af
	dec a
 	jr nz,set_colors
	ld b,a
	ld c,a
	jp #bc38

sound_chords:
	call #bca7
	ld a,1
	ld hl,env
	call #bcbc
	ld de,chords
	setb:
		ld b,20
		halt
		djnz $-1
		ld b,3
		triadloop:
			ld a,(de)
			inc de
			ld (ton+3),a
			ld hl,ton
			push de
			push bc
			call #bcaa
			pop bc
			pop de
			djnz triadloop
		ld a,(ton)
		srl a
		ld (ton),a
		jr nz,setb
	ret

anim_logo:
ld de,morph
call drawlogo
ld de,leosoft
call drawlogo

lionanim1:
ld ix,scraddr_lion
ld hl,scraddr_lion-#800
ld a,1
	lionmovement:
	call #bd19
	push af
	ld b,a
	call #bc29
	push hl
	ld de,lion_decompressed
		lionlines:
		push hl
		ex de,hl
		ld a,b
		ld bc,32
		ldir
		ld b,a
		ex de,hl
		pop hl
		call #bc26
		djnz lionlines
	pop hl
	pop af
	inc a
	cp 63
	jr nz,lionmovement
ret

write_presents:
	ld hl,#1118:call #BB75	;locate 17,24
	ld a,2:call #BB90	;pen 2
	ld a,1:call #BB9F	;transparent
	ld b,8:ld hl,txt_presents
	write_loop:
		ld a,(hl)
		inc hl
		call #BB5A
		djnz write_loop
ret

txt_presents:
db "presents"


morph:
db #02,#00,#56,#56,#56,#00,#00
db #00,#00,#00,#00,#00,#00,#00
db #11,#11,#43,#43,#43,#11,#21
leosoft:
db #02,#01,#56,#56,#56,#10,#10
db #02,#01,#00,#46,#00,#10,#20
db #11,#11,#43,#43,#43,#21,#21


ton:
db 4,1,0,0,0
db 0,15,70,0

env:
db 1,15,-1,8

chords:
db 159,169,253	;bass
db 106,95,84	;mid
db 63,56,50	;treble

outlist:
db 6,25,1,32,2,#2a,7,#1e

colors_leosoft:
db 3,15,25,0


wait:
ld a,180
halt
dec a
jr nz,$-2
ret

parts:
db #EF,#EF,#EF,#EF,#EF,#EF,#EF,#EF
db #EF,#EF,#EF,#EF,#EF,#EF,#EF,#00
db #00,#00,#00,#00,#00,#00,#00,#00
db #EF,#EF,#CF,#CF,#8F,#0F,#0C,#00
db #EF,#EF,#E7,#E7,#E3,#E1,#60,#00
db #60,#E1,#E3,#E7,#E7,#EF,#EF,#EF
db #0C,#0F,#8F,#CF,#CF,#EF,#EF,#EF

colors_title:
db 26,13,24,20,11,2,1,3,6,16,9,4,12,8,18,0

lion_lzsa:
incbin "lioncomp.bin"
